<?xml version="1.0" encoding="UTF-8"?>
<gui name="muon"
     version="1"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">
    <MenuBar>
        <Menu name="file" >
            <Action name="open_markings" />
            <Separator/>
            <Action name="save_markings" />
            <Action name="save_package_list" />
            <Separator/>
            <Action name="save_download_list" />
            <Action name="download_from_list" />
            <Action name="load_archives" />
            <Separator/>
            <Action name="quit" />
        </Menu>
        <Menu name="edit" >
            <Action name="undo" />
            <Action name="redo" />
            <Action name="revert" />
            <Separator/>
            <Action name="update" />
            <Action name="safeupgrade" />
            <Action name="fullupgrade" />
            <Action name="autoremove" />
            <Separator/>
            <Action name="preview" />
            <Action name="apply" />
        </Menu>
        <Menu name="view" >
            <Action name="history" />
        </Menu>
        <Menu name="settings">
          <Separator />
          <Action name="software_properties" />
        </Menu>
    </MenuBar>
    <ToolBar name="mainToolBar">
        <Action name="preview" />
        <Action name="apply" />
        <Separator/>
        <Action name="update" />
        <Action name="fullupgrade" />
        <Separator/>
        <Action name="undo" />
        <Action name="redo" />
        <Action name="revert" />
    </ToolBar>
</gui>
