<?xml version="1.0" encoding="UTF-8"?>
<gui name="muon-discover"
     version="3"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">
    <ToolBar name="mainToolBar">
        <Action name="back" />
        <Separator/>
        <Action name="discover" />
        <Action name="installed" />
        <Action name="sources" />
    </ToolBar>
</gui>
